// -*- C++ -*-
//
// This file is part of HepMC
// Copyright (C) 2014-2023 The HepMC collaboration (see AUTHORS for details)
//
/**
 *  @file HepMC3.h
 *  @brief Main header
 *
 */
#ifndef HEPMC3_HEPMC3_H
#define HEPMC3_HEPMC3_H

#include "HepMC3/Version.h"

#include "HepMC3/GenEvent.h"
#include "HepMC3/GenParticle.h"
#include "HepMC3/GenVertex.h"

#include "HepMC3/Print.h"
/**
 * @brief HepMC3 main namespace.
 *
 * HepMC3 main namespace
 *
 * @ingroup HepMC3
 */
namespace HepMC3 {
}
#endif
